/*
 * $Id: ball.c,v 1.3 2002/05/06 09:59:43 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle ball.
 *
 * @version $Revision: 1.3 $
 */
#include <sys/bios.h>
#include <stdlib.h>

#include "putipati.h"
#include "ball.h"
#include "man.h"
#include "frag.h"
#include "dispmng.h"
#include "gamemng.h"
#include "stagemng.h"
#include "soundmng.h"
#include "vector.h"
#include "sincos.h"

static Ball ball[BALL_MAX];

#define BALL_SPC_MAX 3

/** ball data indexies */
static int ballIdx[BALL_SPC_MAX];
static int ballIdxMax[BALL_SPC_MAX] = {BALL1_MAX, BALL2_MAX, BALL3_MAX};
static int ballIdxStt[BALL_SPC_MAX] = {0, BALL1_MAX, BALL1_MAX+BALL2_MAX};
static int ballIdxSpr[BALL_SPC_MAX] = 
{BALL_SPRITE_IDX, BALL_SPRITE_IDX+BALL1_MAX, BALL_SPRITE_IDX+BALL1_MAX+BALL2_MAX*4};
static int ballSprSize[BALL_SPC_MAX] = {1, 4, 9};

void initBalls() {
  int i;
  for ( i=0 ; i<BALL_MAX ; i++ ) {
    ball[i].color = NOT_EXIST;
    if ( i < ballIdxStt[1] ) {
      ball[i].rdSpc = 0;
      ball[i].sprIdx = ballIdxSpr[0]+i;
    } else if ( i < ballIdxStt[2] ) {
      ball[i].rdSpc = 1;
      ball[i].sprIdx = ballIdxSpr[1]+(i-ballIdxStt[1])*ballSprSize[1];
    } else {
      ball[i].rdSpc = 2;
      ball[i].sprIdx = ballIdxSpr[2]+(i-ballIdxStt[2])*ballSprSize[2];
    }
  }
  for ( i=0 ; i<BALL_SPC_MAX ; i++ ) {
    ballIdx[i] = ballIdxMax[i];
  }
}

int ballNum;

void clearBalls() {
  int i, j;
  Ball *bl;
  for ( j=0 ; j<BALL_MAX ; j++ ) {
    if ( ball[j].color != NOT_EXIST ) {
      bl = &(ball[j]);
      bl->color = NOT_EXIST;
      for ( i=bl->sprIdx ; i<bl->sprIdx+ballSprSize[bl->rdSpc] ; i++ ) {
	sprite_set_location(i, DISAP_POS, DISAP_POS);
      }
    }
  }
  ballNum = 0;
}

void addBall(int color, int rdSpc, long x, long y, int mx, int my) {
  int i, j;
  Ball *bl;
  int sIdx, sPtn;
  for ( i=0 ; i<ballIdxMax[rdSpc] ; i++ ) {
    ballIdx[rdSpc]--; if ( ballIdx[rdSpc] < 0 ) ballIdx[rdSpc] = ballIdxMax[rdSpc]-1;
    bl = &(ball[ballIdxStt[rdSpc] + ballIdx[rdSpc]]);
    if ( bl->color == NOT_EXIST ) {
      ballNum++;
      bl->pos.x = x; bl->pos.y = y;
      bl->vel.x = mx; bl->vel.y = my;
      bl->color = color;
      bl->radius = (4*(rdSpc+1))<<8; 
      bl->velMxL = bl->velMxR = 0;
      bl->whCnt = 0;

      sIdx = bl->sprIdx;
      switch ( rdSpc ) {
      case 0:
	sPtn = 0x0004;
	break;
      case 1:
	sPtn = 0x0008;
	break;
      case 2:
	sPtn = 0x0014;
	break;
      default:
	sPtn = 0x0004;
	break;
      }
      sPtn += color*0x0400;
      for ( j=0 ; j<ballSprSize[rdSpc] ; j++, sIdx++, sPtn++ ) {
	sprite_set_char(sIdx, sPtn);
      }
      return;
    }
  }
}

/* A ball is smashed. */
static void disapBall(Ball *bl) {
  int i;
  for ( i=bl->sprIdx ; i<bl->sprIdx+ballSprSize[bl->rdSpc] ; i++ ) {
    sprite_set_location(i, DISAP_POS, DISAP_POS);
  }
  for ( i=0 ; i<3*bl->rdSpc+randN(4)+5 ; i++ ) {
    addFrag(bl->color, bl->rdSpc, bl->pos.x, bl->pos.y);
  }
  score += (bl->rdSpc+1)*10;
  if ( prsdBallNum > 0 ) {
    if ( prsdBallColor != bl->color ) {
      prsdBallNum = 0;
      prsdBallBonus = BALL_BONUS_START;
      prsdBallBonusCnt = 0;
      screen_fill_char(SCREEN1, SCREEN_TEXT_WIDTH-1, 1, 1, 3, 0);
    }
  }
  prsdBallNum++;
  screen_fill_char(SCREEN1, SCREEN_TEXT_WIDTH-1, prsdBallNum, 1, 1, 4+0x200*8+bl->color*0x0400);
  prsdBallColor = bl->color;
  if ( prsdBallNum >= 3 ) {
    setBonus(prsdBallBonus, bl->pos.x, bl->pos.y);
    score += prsdBallBonus;
    prsdBallNum = 0;
    screen_fill_char(SCREEN1, SCREEN_TEXT_WIDTH-1, 1, 1, 3, 0);
    playSe(BONUS_SE_NUM+prsdBallBonusCnt);
    if ( prsdBallBonus < BALL_BONUS_MAX ) {
      prsdBallBonus <<= 1;
      prsdBallBonusCnt++;
    }
  } else {
    playSe(PRESS_SE_NUM);
  }
  drawScore();

  bl->color = NOT_EXIST;
  ballNum--;
}

static int checkBallWallHit(Ball *bl) {
  int wx, wy, cx, cy;
  cy = (bl->pos.y>>11)-1;
  for ( wy=0 ; wy<2+bl->rdSpc ; wy++, cy++ ) {
    if ( cy < 0 ) cy = 0;
    else if ( cy >= STAGE_HEIGHT ) cy = STAGE_HEIGHT-1;
    cx = bl->pos.x>>11;
    for ( wx=0 ; wx<2+bl->rdSpc ; wx++, cx++ ) {
      if ( cx < 0 ) cx = 0;
      else if ( cx >= STAGE_WIDTH ) cx = STAGE_WIDTH-1;
      if ( block[cx][cy] ) return 1;
    }
  }
  return 0;
}

/* Take a ball out a wall. */
static void moveOutOfWall(Ball *bl) {
  int r, d = 32;
  long bx = bl->pos.x, by = bl->pos.y;
  bl->vel.x >>= 1; bl->vel.y >>= 1;
  for ( r=1 ; r<24 ; r++, d+=64 ) {
    d &= 255;
    bl->pos.x = bx+(sctbl[d]*r); bl->pos.y = by+(sctbl[d+64]*r);
    if ( !checkBallWallHit(bl) ) return;
  }
  bl->pos.x = (SCREEN_HEIGHT<<7);
  bl->pos.y = (SCREEN_WIDTH<<7);
}

static int ballDstForce[3] = {1480, 1720, 2100};

/* Detect the collision of a ball and men. */
static void checkBallHitMen(Ball *ball) {
  int i, r, r2;
  Man *mn;
  for ( i=0 ; i<MAN_MAX ; i++ ) {
    mn = &(man[i]);
    r = ball->radius; r2 = r<<1;
    if ( ball->pos.y > mn->pos.y || ball->pos.y < mn->pos.y-MAN_LENGTH-r2 ) continue;
    if ( ball->prvPos.x < mn->pos.x+(MAN_WIDTH>>1)-r ) {
      if ( ball->pos.x > mn->pos.x-r2 ) {
	if ( ball->vel.x > 0 ) {
	  ball->vel.x = -ball->vel.x;
	  ball->vel.y -= (ball->vel.y>>2);
	}
	if ( mn->vel.x < 0 ) {
	  ball->vel.x += (mn->vel.x>>1);
	  ball->velMxL += (mn->vel.x>>1);
	}
	if ( ball->velMxR > 0 ) {
	  mn->pos.x = ball->pos.x + r2;
	  mn->vel.x += ball->velMxR;
	  if ( (ball->velMxR - ball->velMxL) > ballDstForce[ball->rdSpc] ) {
	    disapBall(ball);
	    return;
	  }
	} else {
	  ball->pos.x = mn->pos.x - r2;
	}
      }
    } else {
      if ( ball->pos.x < mn->pos.x+MAN_WIDTH ) {
	if ( ball->vel.x < 0 ) {
	  ball->vel.x = -ball->vel.x;
	  ball->vel.y -= (ball->vel.y>>2);
	}
	if ( mn->vel.x > 0 ) {
	  ball->vel.x += (mn->vel.x>>1);
	  ball->velMxR += (mn->vel.x>>1);
	}
	if ( ball->velMxL < 0 ) {
	  mn->pos.x = ball->pos.x - MAN_WIDTH;
	  mn->vel.x += ball->velMxL;
	  if ( (ball->velMxR-ball->velMxL) >  ballDstForce[ball->rdSpc] ) {
	    disapBall(ball);
	    return;
	  }
	} else {
	  ball->pos.x = mn->pos.x + MAN_WIDTH;
	}
      }
    }
  }
}


static long l;
static Vector ofs;
static Vector *vel;
static Vector csft1, csft2;
static long ax, ay;

/* Detect the collision of balls. */
static void checkBallsHit(Ball *ball1, Ball *ball2) {
  ball2->pos.x += ball2->radius;
  ball2->pos.y += ball2->radius;
  ofs.x = ball1->pos.x - ball2->pos.x;
  ofs.y = ball1->pos.y - ball2->pos.y;

  ax = abs(ofs.x); ay = abs(ofs.y);
  if ( ax > ay ) {
    l =  ax + (ay>>1) + 1;
  } else {
    l =  ay + (ax>>1) + 1;
  }

  if ( l <= (ball1->radius+ball2->radius) ) {
    vel = &(ball1->vel);
    csft1 = vctGetElement(vel, &ofs);
    vctSub(vel, &csft1);
    vel = &(ball2->vel);
    csft2 = vctGetElement(vel, &ofs);
    vctSub(vel, &csft2);

    vctMul(&csft1, (ball1->radius<<8)/ball2->radius);
    vctMul(&csft2, (ball2->radius<<8)/ball1->radius);

    if ( ball2->whCnt < 2 ) vctAdd(&(ball2->vel), &csft1);
    if ( ball1->whCnt < 2 ) vctAdd(&(ball1->vel), &csft2);

    vctMul(&ofs, ((ball1->radius+ball2->radius)<<7)/l);
    vctAdd(&(ball1->pos), &(ball2->pos));
    ball1->pos.x >>= 1; ball1->pos.y >>= 1; 
    ball2->pos = ball1->pos;
    vctSub(&(ball2->pos), &ofs);
    if ( checkBallWallHit(ball2) ) ball2->pos = ball2->prvPos;
    vctAdd(&(ball1->pos), &ofs);
    if ( checkBallWallHit(ball1) ) ball1->pos = ball1->prvPos;
  }
  ball2->pos.x -= ball2->radius;
  ball2->pos.y -= ball2->radius;
}

void moveBalls() {
  Ball *bl;
  int i, j;
  long tx;
  int whf;
  int sx, sy, sIdx;
  for ( i=0 ; i<BALL_MAX ; i++ ) {
    if ( ball[i].color == NOT_EXIST ) continue;
    bl = &(ball[i]);
    bl->prvPos = bl->pos;

    whf = 0;
    bl->pos.x += bl->vel.x;
    if ( checkBallWallHit(bl) ) {
      if ( bl->whCnt == 0 ) {
	bl->vel.x = -bl->vel.x;
	bl->pos.x = bl->prvPos.x + bl->vel.x;
	if ( bl->vel.x < 0 ) {
	  bl->velMxL += (bl->vel.x>>1);
	} else {
	  bl->velMxR += (bl->vel.x>>1);
	}
      }
      whf = 1;
    }
    tx = bl->pos.x; bl->pos.x = bl->prvPos.x;
    bl->pos.y += bl->vel.y;
    if ( checkBallWallHit(bl) ) {
      if ( bl->whCnt == 0 ) {
	bl->vel.y = -bl->vel.y;
	bl->pos.y = bl->prvPos.y + bl->vel.y;
      }
      whf = 1;
    }
    bl->pos.x = tx;
    if ( whf ) {
      bl->whCnt++;
      if ( bl->whCnt > 9 ) {
	moveOutOfWall(bl);
      } else if ( (bl->whCnt&3) == 0 ) {
	bl->vel.x = -bl->vel.x;
	bl->vel.y = -bl->vel.y;
	for ( j=0 ; j<bl->whCnt ; j++ ) {
	  vctAdd(&(bl->pos), &(bl->vel));
	}
      }
    } else {
      bl->whCnt = 0;
    }

    checkBallHitMen(bl);
    if ( ball[i].color == NOT_EXIST ) continue;

    bl->pos.x += bl->radius;
    bl->pos.y += bl->radius;
    for ( j=i+1 ; j<BALL_MAX ; j++ ) {
      if ( ball[j].color == NOT_EXIST ) continue;
      checkBallsHit(bl, &ball[j]);
    }
    bl->pos.x -= bl->radius;
    bl->pos.y -= bl->radius;

    if ( bl->pos.x < -(bl->radius<<1) ) bl->pos.x = (SCREEN_HEIGHT<<8);
    else if ( bl->pos.x > (SCREEN_HEIGHT<<8) ) bl->pos.x = -(bl->radius<<1);
    if ( bl->pos.y < -(bl->radius<<1)+(8<<4) ) bl->pos.y = (SCREEN_WIDTH<<8)-(8<<4);
    else if ( bl->pos.y > (SCREEN_WIDTH<<8)-(8<<4) ) bl->pos.y = -(bl->radius<<1)+(8<<4);

    bl->velMxL -= (bl->velMxL>>4);
    bl->velMxR -= (bl->velMxR>>4);
    if ( vctSize(&bl->vel) > 1024 ) {
      bl->vel.x -= (bl->vel.x>>4);
      bl->vel.y -= (bl->vel.y>>4);
      if ( vctSize(&bl->vel) > 10240 ) {
	bl->vel.x = bl->vel.y = 0;
      }
    }

    sx = bl->pos.y>>8; sy = bl->pos.x>>8;
    sIdx = bl->sprIdx;
    switch ( bl->rdSpc ) {
    case 0:
      sprite_set_location(sIdx, sx, sy);
      break;
    case 1:
      sprite_set_location(sIdx  , sx, sy);
      sprite_set_location(sIdx+1, sx+8, sy);
      sprite_set_location(sIdx+2, sx, sy+8);
      sprite_set_location(sIdx+3, sx+8,sy+8);
      break;
    case 2:
      sprite_set_location(sIdx  , sx, sy);
      sprite_set_location(sIdx+1, sx+8, sy);
      sprite_set_location(sIdx+2, sx+16, sy);
      sprite_set_location(sIdx+3, sx, sy+8);
      sprite_set_location(sIdx+4, sx+8, sy+8);
      sprite_set_location(sIdx+5, sx+16, sy+8);
      sprite_set_location(sIdx+6, sx, sy+16);
      sprite_set_location(sIdx+7, sx+8, sy+16);
      sprite_set_location(sIdx+8, sx+16, sy+16);
      break;
    }
  }
}
