/*
 * $Id: dispmng.c,v 1.2 2002/06/04 12:57:42 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle screen.
 *
 * @version $Revision: 1.2 $
 */
#include <sys/bios.h>
#include <sys/libwwc.h>
#include <stdlib.h>

#include "putipati.h"
#include "dispmng.h"
#include "images.h"
#include "frag.h"

static int red_plt[] = {
  0xfff, 0xf77, 0xf66, 0xf66, 0xf55, 0xf55, 0xf44, 0xf44,
  0xf33, 0xf33, 0xf22, 0xf22, 0xf11, 0xf11, 0xf00,
};
static int blue_plt[] = {
  0xfff, 0x77f, 0x66f, 0x66f, 0x55f, 0x55f, 0x44f, 0x44f,
  0x33f, 0x33f, 0x22f, 0x22f, 0x11f, 0x11f, 0x00f,
};
static int green_plt[] = {
  0xfff, 0x7f7, 0x6f6, 0x6f6, 0x5f5, 0x5f5, 0x4f4, 0x4f4,
  0x3f3, 0x3f3, 0x2f2, 0x2f2, 0x1f1, 0x1f1, 0x0f0,
};
static int redd_plt[] = {
  0xfff, 0x877, 0x866, 0x866, 0x855, 0x855, 0x844, 0x844,
  0x833, 0x833, 0x822, 0x822, 0x811, 0x811, 0x800,
};
static int blued_plt[] = {
  0xfff, 0x778, 0x668, 0x668, 0x558, 0x558, 0x448, 0x448,
  0x338, 0x338, 0x228, 0x228, 0x118, 0x118, 0x008,
};
static int greend_plt[] = {
  0xfff, 0x787, 0x686, 0x686, 0x585, 0x585, 0x484, 0x484,
  0x383, 0x383, 0x282, 0x282, 0x181, 0x181, 0x080,
};
static int blw_plt[] = {
  0xfff, 0xfd7, 0xfc6, 0xfb6, 0xfa5, 0xf95, 0xf84, 0xf74,
  0xf63, 0xf53, 0xf42, 0xf32, 0xf21, 0xf11, 0xa44,
};
static int lt_plt[] = {
  0xfff, 0xfa5, 0x631, 0xf66, 0xf55, 0xf55, 0xf44, 0xf44,
  0xf33, 0xf33, 0xf22, 0xf22, 0xf11, 0xf11, 0xf00,
};
static int man_plt[] = {
  0xfff, 0xad7, 0xac6, 0xab6, 0xaa5, 0xa95, 0xa84, 0xa74,
  0xa63, 0xa53, 0xa42, 0xa32, 0xa21, 0xa11, 0x633,
};

/**
 * Initialize WonderSwan color display.
 */
void initDisplay() {
  int i;
  lcd_set_segments(0x2);
  display_control(0x0007);
  screen_set_scroll(SCREEN1, 0, 0);
  screen_set_scroll(SCREEN2, 0, 0);
  clearDisplay();
  if ( wwc_get_hardarch() == HARDARCH_WSC ) {
    wwc_set_color_mode(COLOR_MODE_16PACKED);
    wwc_palette_set_color(0, 0, 0xfff);
    for ( i=0 ; i<15 ; i++ ) {
      wwc_palette_set_color(1, i, blw_plt[i]);
      wwc_palette_set_color(8, i, red_plt[i]);
      wwc_palette_set_color(9, i, redd_plt[i]);
      wwc_palette_set_color(10, i, blue_plt[i]);
      wwc_palette_set_color(11, i, blued_plt[i]);
      wwc_palette_set_color(12, i, green_plt[i]);
      wwc_palette_set_color(13, i, greend_plt[i]);
      wwc_palette_set_color(14, i, lt_plt[i]);
      wwc_palette_set_color(15, i, man_plt[i]);
    }
  } else {
    while ( key_press_check() );
    text_set_mode(TEXT_MODE_ANK);
    text_screen_init();
    text_put_string(2, 8, "This game works with");
    text_put_string(10, 9, "WonderSwan Color");
    while ( !key_press_check() );
    loopf = FALSE;
    return;
  }
  sprite_set_range(0, SPRITE_RANGE);
  for ( i=0 ; i<SPRITE_RANGE ; i++ ) {
    sprite_set_location(i, DISAP_POS, DISAP_POS);
  }

  wwc_font_set_colordata(0, 1, img_space);
  wwc_font_set_colordata(4, 1, img_ball1);
  wwc_font_set_colordata(8, 4, img_ball2);
  wwc_font_set_colordata(20, 9, img_ball3);
  wwc_font_set_colordata(32, 1, img_frag1);
  wwc_font_set_colordata(64, 1, img_wall0);
  wwc_font_set_colordata(65, 1, img_wall1);
  wwc_font_set_colordata(66, 1, img_wall2);
  wwc_font_set_colordata(67, 1, img_wall3);
  wwc_font_set_colordata(68, 1, img_wall4);
  wwc_font_set_colordata(69, 1, img_wall5);
  wwc_font_set_colordata(70, 1, img_wall6);
  wwc_font_set_colordata(71, 1, img_space);
  wwc_font_set_colordata(92, 1, img_spot0);
  wwc_font_set_colordata(93, 1, img_spot1);
  wwc_font_set_colordata(94, 1, img_spot2);
  wwc_font_set_colordata(95, 1, img_spot3);
  wwc_font_set_colordata(96, 1, img_spot4);
  wwc_font_set_colordata(97, 1, img_spot5);
  wwc_font_set_colordata(98, 1, img_spot6);
  wwc_font_set_colordata(99, 1, img_spot7);
  wwc_font_set_colordata(100, 1, img_spot8);
  wwc_font_set_colordata(101, 1, img_spot9);
  wwc_font_set_colordata(102, 1, img_spot10);
  wwc_font_set_colordata(103, 1, img_spot11);
  wwc_font_set_colordata(104, 1, img_spot12);
  wwc_font_set_colordata(105, 1, img_spot13);
  wwc_font_set_colordata(106, 1, img_spot14);
  wwc_font_set_colordata(107, 1, img_spot15);
  wwc_font_set_colordata(112, 1, img_brs);
  wwc_font_set_colordata(128, 26, img_alp);
  wwc_font_set_colordata(154, 10, img_num);
  wwc_font_set_colordata(164, 2, img_arrow);
  wwc_font_set_colordata(256, 1, img_man0);
  wwc_font_set_colordata(257, 1, img_man1);
  wwc_font_set_colordata(258, 1, img_man2);
  wwc_font_set_colordata(259, 1, img_man3);
  wwc_font_set_colordata(260, 1, img_man4);
  wwc_font_set_colordata(272, 64, img_title);

  sprite_set_char(MAN_SPRITE_IDX, 256+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+1, 257+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+2, 257+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+3, 257+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+4, 258+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+5, 259+0x0200*0);

  sprite_set_char(MAN_SPRITE_IDX+6, 256+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+7, 257+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+8, 257+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+9, 257+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+10, 258+0x0200*7);
  sprite_set_char(MAN_SPRITE_IDX+11, 260+0x0200*2);

  sprite_set_char(BONUS_SPRITE_IDX+2, NUM_SPR_CHAR);
  sprite_set_char(BONUS_SPRITE_IDX+3, NUM_SPR_CHAR);
}

void closeDisplay() {
  if ( wwc_get_hardarch() == HARDARCH_WSC ) {
    wwc_set_color_mode(COLOR_MODE_GRAYSCALE);
  }
}

void clearDisplay() {
  screen_fill_char(SCREEN1, 0, 0, SCREEN_TEXT_WIDTH, SCREEN_TEXT_HEIGHT, 0x0000);
  screen_fill_char(SCREEN2, 0, 0, SCREEN_TEXT_WIDTH, SCREEN_TEXT_HEIGHT, 0x0000);
}

/**
 * Draw character patterns.
 */
void drawAlp(char c, int x, int y) {
  screen_fill_char(SCREEN1, x, y, 1, 1, ALP_CHAR+(c-'a'));
}

void drawAlp2(char c, int x, int y) {
  if ( c == ' ' ) return;
  screen_fill_char(SCREEN2, x, y, 1, 1, ALP_CHAR+(c-'a'));
}

void drawNum(long n, int x, int y) {
  if ( n == 0 ) {
    screen_fill_char(SCREEN1, x, y, 1, 1, NUM_CHAR);
  } else {
    while ( n > 0 ) {
      screen_fill_char(SCREEN1, x, y, 1, 1, n%10+NUM_CHAR);
      n /= 10;
      y--;
    }
  }
}

void drawNum2(long n, int x, int y) {
  if ( n == 0 ) {
    screen_fill_char(SCREEN2, x, y, 1, 1, NUM_CHAR);
  } else {
    while ( n > 0 ) {
      screen_fill_char(SCREEN2, x, y, 1, 1, n%10+NUM_CHAR);
      n /= 10;
      y--;
    }
  }
}
