/*
 * $Id: frag.c,v 1.2 2002/05/04 15:33:35 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle inks and bonus display.
 *
 * @version $Revision: 1.2 $
 */
#include <sys/bios.h>
#include <stdlib.h>

#include "putipati.h"
#include "frag.h"
#include "stagemng.h"
#include "dispmng.h"

static Frag frag[FRAG_MAX];
static Bonus bonus;

void initFrags() {
  int i;
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    frag[i].cnt = -1;
  }
  bonus.cnt = -1;
}

void clearFrags() {
  int i;
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    sprite_set_location(FRAG_SPRITE_IDX+i, DISAP_POS, DISAP_POS);
  }
  for ( i=0 ; i<4 ; i++ ) {
    sprite_set_location(BONUS_SPRITE_IDX+i, DISAP_POS, DISAP_POS);
  }
}

static int frIdx = FRAG_MAX;

void addFrag(int color, int rdSpc, long x, long y) {
  Frag *fr;
  frIdx--; if ( frIdx < 0 ) frIdx = FRAG_MAX-1;
  fr = &(frag[frIdx]);
  fr->stPos.x = (x>>11); fr->stPos.y = (y>>11);
  fr->pos.x = x; fr->pos.y = y;
  fr->vel.x = randNS(2048); fr->vel.y = randNS(2048); 
  fr->rdSpc = rdSpc;
  fr->color = color;
  fr->cnt = 3 + randN(10);
  sprite_set_char(frIdx + FRAG_SPRITE_IDX, 32+0x400*color);
}

void setBonus(int score, long x, long y) {
  int s;
  bonus.pos.x = x-(12<<8); bonus.pos.y = y;
  bonus.vel.y = 1024;
  bonus.score = score;
  bonus.cnt = 48;
  if ( score >= 1000 ) {
    sprite_set_char(BONUS_SPRITE_IDX, NUM_SPR_CHAR+1);
    s = score - 1000;
  } else {
    sprite_set_char(BONUS_SPRITE_IDX, 0);
    s = score;
  }
  sprite_set_char(BONUS_SPRITE_IDX+1, NUM_SPR_CHAR+s/100);
}

void moveFrags() {
  int i, j;
  int sx, sy, x, y;
  Frag *fr;
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    if ( frag[i].cnt < 0 ) continue;
    fr = &(frag[i]);
    fr->pos.x += fr->vel.x;
    fr->pos.y += fr->vel.y;
    sprite_set_location(FRAG_SPRITE_IDX+i, (fr->pos.y>>8), (fr->pos.x>>8));
    fr->cnt--;
    if ( fr->cnt < 0 ) {
      sprite_set_location(FRAG_SPRITE_IDX+i, DISAP_POS, DISAP_POS);
      sx = fr->stPos.x+randNS(7); sy = fr->stPos.y+randNS(7);
      for ( j=0 ; j<3+randN(2) ; j++ ) {
	x = sx+randNS(2); y = sy+randNS(2);
	if ( x>=0 && x<SPOT_WIDTH && y>=0 && y<SPOT_HEIGHT ) {
	  addSpot(x, y, fr->color+1);
	}
      }
    }
  }
  if ( bonus.cnt < 0 ) return;
  bonus.pos.y += bonus.vel.y;
  bonus.vel.y -= bonus.vel.y>>2;
  bonus.cnt--;
  x = bonus.pos.x>>8;
  if ( bonus.cnt >= 0 ) {
    for ( i=0 ; i<4 ; i++, x+=8 ) {
      sprite_set_location(BONUS_SPRITE_IDX+i, (bonus.pos.y>>8), x);
    }
  } else {
    for ( i=0 ; i<4 ; i++, x+=8 ) {
      sprite_set_location(BONUS_SPRITE_IDX+i, DISAP_POS, DISAP_POS);
    }
  }
}
