/*
 * $Id: gamemng.c,v 1.2 2002/05/05 15:15:02 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle game status.
 *
 * @version $Revision: 1.2 $
 */
#include <sys/bios.h>
#include <stdlib.h>

#include "putipati.h"
#include "gamemng.h"
#include "ball.h"
#include "man.h"
#include "frag.h"
#include "dispmng.h"
#include "stagemng.h"
#include "soundmng.h"
#include "attrcmng.h"

#include "stagedta.h"

GameStatus gameStatus;

/* button state */
Boolean aPrsd = TRUE;
Boolean startPrsd = TRUE;

/* stage data */
int stage;
int stageTime, stageTimeStart;

/* score, bonus data */
long score;
int prsdBallNum, prsdBallColor, prsdBallBonus, prsdBallBonusCnt;

void drawScore() {
  drawNum(score, SCREEN_TEXT_WIDTH-1, SCREEN_TEXT_HEIGHT-2);
}

void startGame() {
  gameStatus.status = GAME_IN_PROGRESS;
  clearDisplay();
  initStage();
  initMen();
  initBalls();
  initFrags();
  stage = -1;
  score = 0;
  prsdBallNum = 0;
  prsdBallBonus = BALL_BONUS_START;
  prsdBallBonusCnt = 0;
  screen_fill_char(SCREEN1, SCREEN_TEXT_WIDTH-1, 0, 1, SCREEN_TEXT_HEIGHT, 0);
  drawScore();

  gotoNextStage();
}

void gotoNextStage() {
  clearBalls();

  stage++;
  if ( stage >= STAGE_NUM ) {
    gotoGameover();
    return;
  }

  initMen();
  setStage(stage);
  gameStatus.atrcStat = STAGE_START;
  gameStatus.atrcTime = 0;
  startStageMessage(stage+1);

  playBgm(0, 1);
}


/**
 * Proceed one frame.
 */
void handleInGame() {
  int key = key_press_check();
  if ( !(key & KEY_START) ) {
    startPrsd = FALSE;
  } else if ( !startPrsd ) { 
    startPrsd = TRUE;
    gotoPause();
    return;
  }
  moveMen(key);
  if ( gameStatus.atrcStat != STAGE_CLEAR ) {
    moveBalls();
    stageTime--;
    if ( ballNum <= 0 ) {
      gameStatus.atrcStat = STAGE_CLEAR;
      startClearMessage(stageTime);
      gameStatus.atrcTime = 0;
      clearBalls();
      fadeBgm();
      timerBlink = 1;
      return;
    } else if ( stageTime <= 0 ) {
      clearBalls();
      clearFrags();
      fadeBgm();
      timerBlink = 1;
      gotoGameover();
      return;
    }
  }
  moveFrags();
  startBrush(key);
  moveBrush();
  switch ( gameStatus.atrcStat ) {
  case STAGE_START:
    gameStatus.atrcTime++;
    printStageMessage(gameStatus.atrcTime<<4);
    if ( gameStatus.atrcTime > 64 ) {
      gameStatus.atrcStat = IN_GAME;
      startTimeMessage(stageTimeStart);
    }
    break;
  case IN_GAME:
    printTimeMessage(stageTime);
    break;
  case STAGE_CLEAR:
    gameStatus.atrcTime++;
    if ( gameStatus.atrcTime > 64 ) {
      gotoNextStage();
    }
  }
}
