/*
 * $Id: stagemng.c,v 1.1.1.1 2002/05/04 09:19:09 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle stage data.
 *
 * @version $Revision: 1.1.1.1 $
 */
#include <sys/bios.h>

#include "putipati.h"
#include "gamemng.h"
#include "stagemng.h"
#include "stagedta.h"
#include "sincos.h"
#include "ball.h"

/* stage blocks */
int block[STAGE_WIDTH][STAGE_HEIGHT];
/* ink spots */
int spot[SPOT_WIDTH][SPOT_HEIGHT];

/* brush data */
static int brushY, brushMy;


/* Handle stage data. */

static int rb[9];

static void setBlockChar(int x, int y) {
  int sx, sy, ex, ey, ssy;
  int c = 0, i;
  for ( i=0 ; i<9 ; i++ ) rb[i] = 0;
  sx = x-1; sy = y-1;
  if ( sx < 0 ) sx = 0;
  if ( sy < 0 ) sy = 0;
  ex = x+1; ey = y+1;
  if ( ex >= STAGE_WIDTH ) ex = STAGE_WIDTH-1;
  if ( ey >= STAGE_HEIGHT ) ey = STAGE_HEIGHT-1;
  ssy = sy;
  for ( ; sx<=ex ; sx++ ) {
    for ( sy=ssy ; sy<=ey ; sy++ ) {
      rb[sx-(x-1) + (sy-(y-1))*3] = block[sx][sy];
    }
  }
  if ( (rb[1] && rb[5]) && ((!rb[3] && !rb[7]) || !rb[2]) ) c = 5;
  else if ( (rb[5] && rb[7]) && ((!rb[1] && !rb[3]) || !rb[8]) ) c = 4;
  else if ( (rb[3] && rb[7]) && ((!rb[1] && !rb[5]) || !rb[6]) ) c = 3;
  else if ( (rb[1] && rb[3]) && ((!rb[5] && !rb[7]) || !rb[0]) ) c = 6;
  else if ( (rb[1] || rb[7]) && (!rb[3] || !rb[5]) ) c = 2;
  else if ( (rb[3] || rb[5]) && (!rb[1] || !rb[7]) ) c = 1;
  screen_fill_char(SCREEN1, y+1, x, 1, 1, BLOCK_CHAR+c);
}

static void unsetBlockChar(int x, int y) {
  screen_fill_char(SCREEN1, y+1, x, 1, 1, BLOCK_CHAR+7);
}

void initStage() {
  int x, y;
  for ( y=0 ; y<STAGE_HEIGHT ; y++ ) {
    for ( x=0 ; x<STAGE_WIDTH ; x++ ) {
      block[x][y] = 0;
    }
  }
  for ( y=0 ; y<SPOT_HEIGHT ; y++ ) {
    for ( x=0 ; x<SPOT_WIDTH ; x++ ) {
      spot[x][y] = 0;
    }
  }
  brushY = -1; brushMy = 1;
}

void setStage(int n) {
  int x, y, i, d, s;
  long mx, my;
  unsigned long bp, sd;
  for ( y=0 ; y<STAGE_HEIGHT ; y++ ) {
    bp = 0x20000; sd = stage_block_data[n][y];
    for ( x=0 ; x<STAGE_WIDTH ; x++ ) {
      if ( sd & bp ) {
	block[x][y] = 1;
      } else {
	block[x][y] = 0;
      }
      bp >>= 1;
    }
  }
  for ( y=0 ; y<STAGE_HEIGHT ; y++ ) {
    for ( x=0 ; x<STAGE_WIDTH ; x++ ) {
      if ( block[x][y] ) {
	setBlockChar(x, y);
      } else {
	unsetBlockChar(x, y);
      }
    }
  }

  for ( i=0 ; i<BALL_DATA_MAX ; i+=6 ) {
    if ( stage_ball_data[n][i] == -1 ) break;
    d = stage_ball_data[n][i+2]; s = stage_ball_data[n][i+3]; 
    mx = sctbl[d]; my = sctbl[d+64]; 
    mx = (mx*s)>>8; my = (my*s)>>8;
    addBall(stage_ball_data[n][i+4], stage_ball_data[n][i+5],
	    ((long)stage_ball_data[n][i]<<11), ((long)stage_ball_data[n][i+1]<<11),
	    (int)mx, (int)my);
  }

  stageTimeStart = stageTime = stage_time[n]*GAME_TIME_INTERVAL;
}


/* Handle spots. */

#define SPOT_CHAR (0x1200+92-0x400)

void setSpotChar(int x, int y) {
  int sb = 0, c;
  c = spot[x][y]; if ( c == 0 ) return;
  if ( y>0 && spot[x][y-1]==c ) sb |= 8;
  if ( x<SPOT_WIDTH-1 && spot[x+1][y]==c ) sb |= 4;
  if ( y<SPOT_HEIGHT-1 && spot[x][y+1]==c ) sb |= 2;
  if ( x>0 && spot[x-1][y]==c ) sb |= 1;
  screen_fill_char(SCREEN2, y, x, 1, 1, SPOT_CHAR+sb+0x0400*c);
}

void addSpot(int x, int y, int c) {
  int sx, sy, ex, ey, ssy;
  spot[x][y] = c;
  sx = x-1; sy = y-1;
  if ( sx < 0 ) sx = 0;
  if ( sy < 0 ) sy = 0;
  ex = x+1; ey = y+1;
  if ( ex >= SPOT_WIDTH ) ex = SPOT_WIDTH-1;
  if ( ey >= SPOT_HEIGHT ) ey = SPOT_HEIGHT-1;
  ssy = sy;
  for ( ; sx<=ex ; sx++ ) {
    for ( sy=ssy ; sy<=ey ; sy++ ) {
      setSpotChar(sx, sy);
    }
  }
}


/* Handle a brush. */

void startBrush(int key) {
  if ( !(key & (KEY_A | KEY_B)) ) {
    aPrsd = FALSE;
  } else if ( !aPrsd ) { 
    aPrsd = TRUE;
    if ( brushY != -1 && brushY != SPOT_HEIGHT ) return;
    brushY += brushMy;
  }
}

#define BRUSH_CHAR (0x0200+112)

void moveBrush() {
  int x;
  if ( brushY == -1 || brushY == SPOT_HEIGHT ) return;
  for ( x=0 ; x<SPOT_WIDTH ; x++ ) {
    spot[x][brushY] = 0;
  }
  screen_fill_char(SCREEN2, brushY, 0, 1, SPOT_WIDTH, 0);
  brushY += brushMy;
  if ( brushY == -1 || brushY == SPOT_HEIGHT ) {
    brushMy = -brushMy;
    return;
  }
  screen_fill_char(SCREEN2, brushY, 0, 1, SPOT_WIDTH, BRUSH_CHAR);
}
